import React from 'react';
import PropTypes from 'prop-types';
import ChartWrapper from '../ChartWrapper/ChartWrapper';

// Import styles
import getStyles from './ScatterBubbleChart.styles';

/**
 * A scatter bubble chart is a type of plot or mathematical diagram using Cartesian coordinates to display values for typically two variables for a set of data.
 * @see https://echarts.apache.org/examples/en/editor.html?c=bubble-gradient
 */
const ScatterBubbleChart = (props) => {
    const {
        animation,
        contextStyles,
        gridLines,
        horizontalGridLines,
        legendText,
        maxX,
        maxY,
        minX,
        minY,
        onPress,
        seriesData,
        testID,
        title,
        tooltip,
        valueLabelFormat,
        verticalGridLines,
        xAxis,
        xAxisTitle,
        yAxis,
        yAxisTitle,
        zoom,
    } = props;

    const styles = getStyles(contextStyles);

    /** Build axis options */
    const axisOptions = {
        xAxis: {
            axisTick: {
                // Move the tick to the center of each label
                alignWithLabel: true,
            },
            show: xAxis,
            splitLine: {
                // Show vertical grid lines
                show: gridLines && verticalGridLines,
            },
        },
        yAxis: {
            show: yAxis,
            splitLine: {
                // Show horizontal grid lines
                show: gridLines && horizontalGridLines,
            },
        },
    };

    // Set axis titles
    if (xAxisTitle) {
        axisOptions.xAxis.name = xAxisTitle;
        axisOptions.xAxis.nameLocation = 'center';
        axisOptions.xAxis.nameGap = 25;
    }
    if (yAxisTitle) {
        axisOptions.yAxis.name = yAxisTitle;
        axisOptions.yAxis.nameLocation = 'center';
        axisOptions.yAxis.nameGap = 30;
    }

    /** Build chart options */
    const chartOptions = {
        animation,
        backgroundColor: 'transparent',
        grid: {
            containLabel: true,
            show: true,
        },
        xAxis: axisOptions.xAxis,
        yAxis: axisOptions.yAxis,
    };

    // Set chart title
    if (title) {
        chartOptions.title = {
            text: title,
        };
    }

    // Enable tooltip
    if (tooltip) {
        chartOptions.tooltip = {
            confine: true,
        };
    }

    // Set chart legend
    if (legendText) { chartOptions.legend = { data: legendText }; }

    // Map series data prop to chart data structure
    const mappedChartData = seriesData.map((dataSeries) => (
        dataSeries.map((item) => {
            // Begin minimum bubble detail
            const bubbleDetail = {
                itemStyle: {},
                name: item.name,
                value: item.value,
            };

            // Add bubble label details
            if (item.valueLabel) {
                bubbleDetail.label = {
                    show: true,
                    formatter: (item.valueLabelFormat || valueLabelFormat).replace('{value}', '{c}'),
                };

                // Apply label styles
                if (item.valueLabelColor) { bubbleDetail.label.color = item.valueLabelColor; }
                if (item.valueLabelSize) { bubbleDetail.label.fontSize = item.valueLabelSize; }

                // Apply label dimensions - NOT Working
                if (item.valueLabelWidth) { bubbleDetail.label.width = item.valueLabelWidth ; }
                if (item.valueLabelHeight) { bubbleDetail.label.height = item.valueLabelHeight ; }
            }

            // Add Bubble size
            if (item.symbolSize) {
                bubbleDetail.symbolSize = item.symbolSize;
            }

            // Add Bubble styles
            if (item.symbolColor) {
                bubbleDetail.itemStyle.color = item.symbolColor;
            }
            if (item.symbolEmphasisColor) {
                bubbleDetail.itemStyle.emphasis = { color: item.symbolEmphasisColor };
            }
            if (item.symbolBorderColor) {
                bubbleDetail.itemStyle.borderColor = item.symbolBorderColor;
            }
            if (item.symbolBorderWidth || item.symbolBorderWidth === 0) {
                bubbleDetail.itemStyle.borderWidth = item.symbolBorderWidth;
            }
            if (item.symbolOpacity || item.symbolOpacity === 0) {
                bubbleDetail.itemStyle.opacity = item.symbolOpacity;
            }

            return bubbleDetail;
        })
    ));

    // Set range parameters
    if (maxX || maxX === 0) { chartOptions.xAxis.max = maxX; }
    if (minX || minX === 0) { chartOptions.xAxis.min = minX; }
    if (maxX || minX) { chartOptions.xAxis.scale = true; }
    if (maxY || maxY === 0) { chartOptions.yAxis.max = maxY; }
    if (minY || minY === 0) { chartOptions.yAxis.min = minY; }
    if (maxY || minY) { chartOptions.yAxis.scale = true; }

    /** Set chart options */
    const seriesArray = mappedChartData.map((series, idx) => {
        const seriesData = {
            type: 'scatter',
            data: series,
        };
        if (legendText) { seriesData.name = legendText[idx]; }
        return seriesData;
    });

    // // Enable zoom
    if (zoom) {
        chartOptions.dataZoom = [
            {
                type: 'slider',
                orient: 'horizontal',
            },
            {
                type: 'inside',
                orient: 'horizontal',
            },
        ];
    }
    chartOptions.series = seriesArray;

    // Generate props
    const chartProps = {
        contextStyles: {
            container: styles.container,
        },
        option: chartOptions,
    };
    if (onPress) { chartProps.onPress = onPress; }
    if (testID) { chartProps.testID = testID; }

    return (
        <ChartWrapper { ...chartProps } />
    );
};

ScatterBubbleChart.propTypes = {
    /** Enables/disables chart animations */
    animation: PropTypes.bool,

    /** Styles for the component */
    contextStyles: PropTypes.shape({
        /** Styles for the container */
        container: PropTypes.object,
    }),

    /** Enable display of grid lines */
    gridLines: PropTypes.bool,

    /** Enable display of horizontal grid lines */
    horizontalGridLines: PropTypes.bool,

    /** Displays a legend symbol with text for the represented bars */
    legendText: PropTypes.arrayOf(PropTypes.string),

    /** Maximum X axis value */
    maxX: PropTypes.number,

    /** Maximum Y axis value */
    maxY: PropTypes.number,

    /** Minimum X axis value */
    minX: PropTypes.number,

    /** Minimum Y axis value */
    minY: PropTypes.number,

    /**
     * Triggered on press of a bar
     * @param {Object} itemData - Data for the bar pressed
     */
    onPress: PropTypes.func,

    /** The data points to display for each bubble */
    seriesData: PropTypes.arrayOf(PropTypes.arrayOf(PropTypes.shape({
        /** The name of the bubble */
        name: PropTypes.string.isRequired,

        /** Border width of the symbol */
        symbolBorderWidth: PropTypes.number,

        /** Border color of the symbol */
        symbolBorderColor: PropTypes.string,

        /** Color of the bubble */
        symbolColor: PropTypes.string,

        /** Emphasis color of the bubble on press/hover */
        symbolEmphasisColor: PropTypes.string,

        /** Opacity of the symbol */
        symbolOpacity: PropTypes.number,

        /** Size of the bubble */
        symbolSize: PropTypes.oneOfType([

            /** Size of the symbol in number */
            PropTypes.number,

            /** Array to represent width and height - ex: [ 10, 20 ] where width is 10 and height is 20 */
            PropTypes.array,
        ]),

        /** The value of the bubble that represents the X and Y coordinates */
        value: PropTypes.arrayOf(PropTypes.number).isRequired,

        /** Display label with the bubble value */
        valueLabel: PropTypes.bool,

        /** Color of the bubble value label */
        valueLabelColor: PropTypes.string,

        /** Font size of the bubble value label */
        valueLabelSize: PropTypes.number,

        /** Height of the value label */
        valueLabelHeight: PropTypes.oneOfType([
            /** Height of the content block */
            PropTypes.number,

            /** Percentage of the content height */
            PropTypes.string,
        ]),

        /** Width of the value label */
        valueLabelWidth: PropTypes.oneOfType([
            /** Width of the content block */
            PropTypes.number,

            /** Percentage of the content width */
            PropTypes.string,
        ]),
    }))).isRequired,

    /** Id used for testing */
    testID: PropTypes.string,

    /** Title text for the chart */
    title: PropTypes.string,

    /** Show related data in a tooltip after pressing a bubble */
    tooltip: PropTypes.bool,

    /**
     * Format string for the value label
     * @see https://echarts.apache.org/en/option.html#series-scatter.label.formatter for the available formatting options
     */
    valueLabelFormat: PropTypes.string,

    /** Enable display of vertical grid lines */
    verticalGridLines: PropTypes.bool,

    /** Show X axis line, ticks and labels */
    xAxis: PropTypes.bool,

    /** Name of xAxis */
    xAxisTitle: PropTypes.string,

    /** Show Y axis line, ticks and labels */
    yAxis: PropTypes.bool,

    /** Name of yAxis */
    yAxisTitle: PropTypes.string,

    /** Enable zooming in/out chart ranges */
    zoom: PropTypes.bool,
};

ScatterBubbleChart.defaultProps = {
    animation: true,
    contextStyles: {},
    gridLines: true,
    horizontalGridLines: true,
    tooltip: true,
    valueLabelFormat: '{value}',
    verticalGridLines: true,
    xAxis: true,
    xAxisTitle: 'X',
    yAxis: true,
    yAxisTitle: 'Y',
    zoom: true,
};

export default ScatterBubbleChart;
