import React from 'react';
import PropTypes from 'prop-types';
import MuiButton from '@material-ui/core/Button';
import { MuiThemeProvider, createMuiTheme } from '@material-ui/core/styles';
import Icon from '../Icon/Icon';
import getStyles from './ButtonWithIcon.styles';

/** Default button variant. 'containted' is as per the Material UI standards. */
const BUTTON_VARIANT_DEFAULT = 'contained';

/** Default icon size. */
const ICON_SIZE_DEFAULT = 'small';

/** Default state of button. i.e. the button is enabled out of the box, unless the 'disabled' prop is set to 'true'. */
const BUTTON_IS_DISABLED = false;

/**
 * Button component: A control is that shows a user a button with a text or icon representation of an action to be performed and allows the user to select the button to run the action.
 * The control is capable of displaying an activity indicator to indicate that the performed action is in progress.
 * @see https://material-ui.com/demos/buttons/
 * @see https://material-ui.com/api/button/
 * @see https://material-ui.com/api/button-base/
 */
const ButtonWithIcon = (props) => {
    const {
        contextStyles,
        disabled,
        iconName,
        variant,
        onClick,
        iconSize,
        testID,
    } = props;

    const styles = getStyles(contextStyles);

    const theme = createMuiTheme({
        typography: {
            // https://material-ui.com/style/typography/#migration-to-typography-v2
            useNextVariants: true,
        },
        overrides: {
            MuiButton: {
                // Name of the rule
                root: {
                    ...styles.buttonText,
                },
                outlined: {
                    ...styles.contentContainer,
                },
                contained: {
                    ...styles.contentContainer,
                },
            },
        },
    });

    const iconStyles = {
        container: styles.iconContainer,
        icon: styles.icon,
    };

    return (
        <div
            style={ styles.container }
            data-testid={ testID && `${testID}__container` }>

            <MuiThemeProvider theme={ theme }>
                <MuiButton
                    variant={ variant }
                    disabled={ disabled }
                    data-testid={ testID && `${testID}__button` }
                    onClick={ onClick }>
                    <Icon
                        iconName={ iconName }
                        contextStyles={ iconStyles }
                        iconSize={ iconSize }
                        data-testid={ testID && `${testID}__icon` }
                        title={ iconName } />
                </MuiButton>
            </MuiThemeProvider>

        </div>
    );
};

ButtonWithIcon.propTypes = {
    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Styling for the main button container */
        container: PropTypes.object,

        /** Styles for the container around the button text and icon */
        contentContainer: PropTypes.object,

        /** Styles for the container around the icon */
        iconContainer: PropTypes.object,

        /** Styles for icon */
        icon: PropTypes.object,
    }),

    /** The icon to show */
    iconName: PropTypes.string.isRequired,

    /** Size of the icon to be rendered. */
    iconSize: PropTypes.oneOf([
        'inherit',
        'default',
        'small',
        'large',
    ]),

    /** Button Variant. Defaults to the 'contained' variant. Valid Types: 'outlined', 'contained' */
    variant: PropTypes.oneOf([
        'contained',
        'outlined',
    ]),

    /** Disable the button */
    disabled: PropTypes.bool,

    /**
     * Called when the button is clicked.
     * @param {Object} props - The button instance props
     * @param {SyntheticEvent} event - The click event
     */
    onClick: PropTypes.func,

    /** Id used for testing */
    testID: PropTypes.string,
};

ButtonWithIcon.defaultProps = {
    contextStyles: {},
    disabled: BUTTON_IS_DISABLED,
    iconSize: ICON_SIZE_DEFAULT,
    variant: BUTTON_VARIANT_DEFAULT,
};

export default ButtonWithIcon;