import React from 'react';
import PropTypes from 'prop-types';
import MuiButton from '@material-ui/core/Button';
import { MuiThemeProvider, createMuiTheme } from '@material-ui/core/styles';
import Icon from '../Icon/Icon';
import TextLabel from '../TextLabel/TextLabel';
import getStyles from './ButtonWithTextAndIcon.styles';

/** Default location for Icon in a button with respect to button text. */
const ICON_LOCATION_DEFAULT = 'left';

/** Default button variant. 'containted' is as per the Material UI standards. */
const BUTTON_VARIANT_DEFAULT = 'contained';

/** Default icon size. */
const ICON_SIZE_DEFAULT = 'small';

/** Default state of button. i.e. the button is enabled out of the box, unless the 'isDisabled' prop is set to 'true'. */
const BUTTON_IS_DISABLED = false;

/**
 * Button component: A control is that shows a user a button with a text or icon representation of an action to be performed and allows the user to select the button to run the action.
 * The control is capable of displaying an activity indicator to indicate that the performed action is in progress.
 * @see https://material-ui.com/demos/buttons/
 * @see https://material-ui.com/api/button/
 * @see https://material-ui.com/api/button-base/
 */
const ButtonWithTextAndIcon = (props) => {
    const {
        contextStyles,
        children,
        text,
        iconName,
        iconLocation,
        disabled,
        variant,
        iconSize,
        onClick,
        testID,
    } = props;

    const styles = getStyles(contextStyles);

    const textStyles = {
        container: styles.buttonTextContainer,
        text: styles.buttonText,
    };

    const theme = createMuiTheme({
        typography: {
            // https://material-ui.com/style/typography/#migration-to-typography-v2
            useNextVariants: true,
        },
        overrides: {
            MuiButton: {
                // Name of the rule
                root: {
                    ...styles.buttonText,
                },
                outlined: {
                    ...styles.contentContainer,
                },
                contained: {
                    ...styles.contentContainer,
                },
            },
        },
    });

    /** Renders the Icon */
    const renderIcon = () => {
        const {
            iconName,
            testID,
        } = props;

        const iconStyles = {
            container: styles.iconContainer,
            icon: styles.icon,
        };

        return (
            <Icon
                iconName={ iconName }
                contextStyles={ iconStyles }
                iconSize={ iconSize }
                data-testid={ testID && `${testID}__icon` }
                title={ iconName } />
        );
    };

    /** If text (or) children are not provided, the container is also not rendered. */
    if (text || children) {
        return (
            <div
                style={ styles.container }
                data-testid={ testID && `${testID}__container` }>

                <MuiThemeProvider theme={ theme }>
                    <MuiButton
                        variant={ variant }
                        disabled={ disabled }
                        data-testid={ testID && `${testID}__button` }
                        onClick={ onClick }>
                        { iconName && iconLocation === 'left' && renderIcon(styles) }
                        <TextLabel
                            data-testid={ testID && `${testID}__textLabel` }
                            contextStyles={ textStyles }>
                            { text || children }
                        </TextLabel>
                        { iconName && iconLocation === 'right' && renderIcon(styles) }
                    </MuiButton>
                </MuiThemeProvider>

            </div>
        );
    }

    return null;
};

ButtonWithTextAndIcon.propTypes = {
    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Styling for the main button container */
        container: PropTypes.object,

        /** Styles for the container around the button text and icon */
        contentContainer: PropTypes.object,

        /** Styles for the container around the button text */
        buttonTextContainer: PropTypes.object,

        /** Styles for the button text */
        buttonText: PropTypes.object,

        /** Styles for the container around the icon */
        iconContainer: PropTypes.object,

        /** Styles for icon */
        icon: PropTypes.object,
    }),

    /** Text displayed inside the button */
    text: PropTypes.string,

    /**
     * Size of the icon to be rendered.
     */
    iconSize: PropTypes.oneOf([
        'inherit',
        'default',
        'small',
        'large',
    ]),

    /** The icon to be displayed */
    iconName: PropTypes.string.isRequired,

    /** Icon location. Defaults to the left. Valid Types: 'left', 'right' */
    iconLocation: PropTypes.oneOf([
        'left',
        'right',
    ]),

    /** Button Variant. Defaults to the 'contained' variant. Valid Types: 'outlined', 'contained' */
    variant: PropTypes.oneOf([
        'contained',
        'outlined',
    ]),

    /** Disable the button */
    disabled: PropTypes.bool,

    /** children for the component */
    children: PropTypes.string,

    /**
     * Called when the button is clicked.
     * @param {Object} props - The button instance props
     * @param {SyntheticEvent} event - The click event
     */
    onClick: PropTypes.func,

    /** Id used for testing */
    testID: PropTypes.string,
};

ButtonWithTextAndIcon.defaultProps = {
    contextStyles: {},
    iconSize: ICON_SIZE_DEFAULT,
    iconLocation: ICON_LOCATION_DEFAULT,
    disabled: BUTTON_IS_DISABLED,
    variant: BUTTON_VARIANT_DEFAULT,
};

export default ButtonWithTextAndIcon;